//libraries
#include <avr/io.h>
#include "lcd.h"
#include <util/delay.h>

#define F_CPU 16000000UL
#define DT PD3
#define SCK PD5

//delay function
void delay(int delay)
{
  for (int i=1; i<=delay; i++)
  _delay_ms(1);
}

//function for readout
long hx711_read() {
    long count = 0;

    //wait for DT to go LOW
    while (PIND & (1 << DT));

    //read 24 bits from HX711
    for (uint8_t i = 0; i < 24; i++)
	{
        PORTD |= (1 << SCK);
        _delay_us(1);
        count = count << 1;

        if (PIND & (1 << DT))
		{
            count++;
        }

        PORTD &= ~(1 << SCK);
        _delay_us(1);
    }

    //25th pulse to set gain = 128 (channel A)
    PORTD |= (1 << SCK);
    _delay_us(1);
    PORTD &= ~(1 << SCK);
    _delay_us(1);

    //convert 24-bit to signed long
    if (count & 0x800000)
	{
        count |= ~0xFFFFFF;
    }

    return count;
}

int main(void)
{
	//set input or output (input is default)
	DDRD &= ~(1 << DT);
    DDRD |= (1 << SCK);
	//make SCK LOW
	PORTD &= ~(1 << SCK);
	
	//initialize LCD (with backlight and no coursor)
	lcd_init();
	lcd_bklt(1);
	lcd_command(0x0C);	
	
	//get zeroed out value
	long weight;
	long init = hx711_read();
	
	//infinite cycle	
    while(1)
    {
		//offset = (value with mass - value without mass) / mass of known object in grams
			//(305320 - 220150) / 100 = 851,7 => 851
		//weight = ((current value - zeroed value) / offset) + manual adjustment
			//mass shown was -3g => add 3 as manual adjustment
		weight = ((hx711_read() - init) / 851) + 3;
		//LCD writeout
		lcd_puts("Hmotnost:");
		lcd_setCursor(1,0);
		char text[]= {"                "};	
		sprintf(text,"%ld",weight);
		lcd_puts(text);
		lcd_puts("g");
		_delay_ms(500);
		lcd_command(0x01);
		_delay_ms(2);
    }
}
//end of program