#include "lcd.h"

/* Primitivne funkcie, nepredpoklada sa ich vyuzitie uzivatelom */

/* Funkcia zapise jeden bajt po SPI zbernici do zariadenia */
void lcd_write( char data )
{
 signed char index = 8;
        char c_data;

 DISPLAY_CSB_PORT &= ~(1<<DISPLAY_CSB_PIN); 	// Chip-Select do log.0 
 c_data = data;

 do
 {
  _delay_us(6);
  if ( c_data & 0x80 )		// najyssi bit zamaskujeme 
	   PORTB |= (1<<3);		// a posleme ho na zbernicu
  else
	   PORTB &= ~(1<<3);    // cez vodic MOSI (alebo len SI)
  _delay_us(5);			    // a vygenerujeme jeden hodinovy pulz
       PORTB &= ~(1<<5);     // na vyvod CLK
  _delay_us(6);
	   PORTB |= (1<<5);    
	
   c_data = c_data << 1;    // na najvyssie miesto posunieme dalsi bit
   index--;

  } while (index > 0);      // a toto spravime dokola 8x

	                        // a napokon zdvihneme /CS do log.1
	_delay_ms( 2 );
	DISPLAY_CSB_PORT |= (1<<DISPLAY_CSB_PIN);   
}


/* Funkcia zapise jeden byte do riadiaceho (Control) registra */
void lcd_command( char instruction )
{
	DISPLAY_RS_PORT &= ~(1<<DISPLAY_RS_PIN);
	_delay_us( 1 );
	lcd_write( instruction );
}


/* Funkcia zapise jeden byte do datoveho (Data) registra */
void lcd_data( char data )
{
	DISPLAY_RS_PORT |=  1<<DISPLAY_RS_PIN;
	_delay_us( 7 );
	lcd_write( data );
}


/* Uzivatelske funkcie, ktore ma pouzivat uzivatel */



/* Inicializacia displeja podla datasheetu pre radic ST7036 */
void lcd_init(void)
{  
	
              DDRB |= (1<<PB3) | (1<<PB5);	   // MOSI + SCK as OUTPUTs
    DISPLAY_RS_DDR |= 1<<DISPLAY_RS_PIN;       // All signals as OUTPUT
   DISPLAY_CSB_DDR |= 1<<DISPLAY_CSB_PIN;
  DISPLAY_BKLT_DDR |= 1<<DISPLAY_BKLT_PIN;
	
     	
             PORTB |= (1<<PB5);               // set_bit( ST7036_CLK );
  DISPLAY_CSB_PORT |= (1<<DISPLAY_CSB_PIN);   // set_bit( ST7036_CSB );
   DISPLAY_RS_PORT &= ~(1<<DISPLAY_RS_PIN);   // set_bit( ST7036_CSB );
  
  _delay_ms(50);		// pockame viac ako 40ms na ustalenie napajacieho napatia

   lcd_command( 0x39 );	// Function set; 8-bit Datenlänge, 2 Zeilen, Instruction table 1
  _delay_us(50);		// mehr als 26,3µs warten

  lcd_command( 0x1d );	// Bias Set; BS 1/5; 3 zeiliges Display /1d
 _delay_us(50);		// mehr als 26,3µs warten

  lcd_command( 0x50 );	// Booster aus; Kontrast C5, C4 setzen /50
 _delay_us(50);		// mehr als 26,3µs warten

  lcd_command( 0x6c );	// Spannungsfolger und Verstärkung setzen /6c
 _delay_ms( 500 );	// mehr als 200ms warten !!!
 
  lcd_command( 0x7c );	// Kontrast C3, C2, C1 setzen /7c
 _delay_us(50);		// mehr als 26,3µs warten

  lcd_command( 0x38 );	// Display EIN, Cursor EIN, Cursor BLINKEN /0f
 _delay_us(50);		// mehr als 26,3µs warten

  lcd_command( 0x0f );	// Display EIN, Cursor EIN, Cursor BLINKEN /0f
  _delay_us(50);		// mehr als 26,3µs warten

  lcd_command( 0x01 );	// Display löschen, Cursor Home
 _delay_ms(400);		//

  lcd_command( 0x06 );	// Cursor Auto-Increment
 _delay_us(50);		// mehr als 26,3µs warten

}

/* Funkcia zobrazi na pozicii kurzoru jeden znak */
/* Alias funkcia z dovodov kompatibility         */
void lcd_putc( char znak )
{
	lcd_data(znak);
}


/* Funkcia zobrazi na displeji nejaky text*/
void lcd_puts(char *string)
{
  while(*string!='\0')
  {
     lcd_data(*string);
     string++;
  } 
}

/* Funkcia nastavi kurzor na poziciu riadok, stlpec */ 
/*   a text sa samozrejme vypise od kurzora dalej.  */
void lcd_setCursor(char row, char col)
{
	unsigned char address;

    if (row == 0)
        address = 0x00 + col;
    else if (row == 1)
        address = 0x10 + col;
	else if (row == 2)
        address = 0x20 + col;
	else if (row == 3)
        address = 0x30 + col;
    else
        return; // Nepodporovaný riadok

    lcd_command(0x80 | address); // Nastav DDRAM adresu
}

void lcd_clearline( unsigned char riadok )
{
 unsigned char index;
 lcd_setCursor( riadok, 0 );
 for (index=1; index<20; index++) lcd_data( ' ' );
}

void lcd_clear( void )
{
	lcd_clearline( 0 );
	lcd_clearline( 1 );
	lcd_clearline( 2 );
}

/* Funkcia zapne alebo vypne podsvietenie displeja */
void lcd_bklt( char OnOff)
{
  if (OnOff) 
     DISPLAY_BKLT_high;
  else
     DISPLAY_BKLT_low;
}

/* Funkcia ulozi uzivatelom definovany znak do CG RAM na Address */
void def_znak(unsigned char *ZnakArray, unsigned char Address)
{
  lcd_command(0x40|(Address<<3));  //nastavenie adresy znaku v CGRAM
  for(unsigned char i = 0;i<8; i++) lcd_data( *(ZnakArray + i));
  lcd_command(0x80);               // zmena na DD RAM
}